#! /usr/bin/env python
# Script to synthesize traces based on time window
# Assumes that the second axis of the files are the indices for the time windows of equal length
# Simply grabs the segments and cats them (no taper, no bandpass)
# Attempts to avoid duplicate files


# import modules needed
import commands
import sys
import os
import re
import random


# some constants (assuming processing is the same, these will not change at the moment)
# need to change these values to fit the stats of the files are that you are reading in
d2=1
n1=240000
o1=0
d1=0.03
tmp_num=random.randint(0,10000)


# parse command line (take care of lower bound of chunk since only looking at one chunk at a time)
def parse_command_line():
  if len(sys.argv) !=4:
    print "Expecting three arguments: directory to search, starting two-hour chunk, and output file name"
  x=re.compile("^R.+\d+\.\d+\.(\d+)\.H$")
  files=[]
  for dirname, dirnames, filenames in os.walk(sys.argv[1]):
    for filename in filenames:
      f=x.search(filename)
      if f:
        if (int(sys.argv[2])>=int(f.group(1))):
          fle= os.path.join(dirname, filename)
          files.append(fle)
  return files,sys.argv[2],sys.argv[3]

# remove duplicates (assume that means files that are at the same receiver point and location
def remove_duplicates(infiles,chunk):
  traces = []
  seen = set()
  for infile in infiles:
    n2=int(commands.getoutput('Get <' + infile + ' n2 parform=n').rstrip().lstrip())
    o2=int(commands.getoutput('Get <' + infile + ' o2 parform=n').rstrip().lstrip())
    e2=int(o2+(n2-1))
    x=re.compile("^.+R(\d+_\d+)\.\d\.\d\.\d+\.H$")
    #x=re.compile("^.+R(\d+_\d+\.\d\.\d\.\d+)\.H$")
    # grab files that meet criteria
    if (int(chunk)<=e2):
      # print file path name
      #print infile
      # search for the coordinates and start time chunk
      f=x.search(infile)
      if f.group(1) not in seen:
        traces.append(infile)
        seen.add(f.group(1))
  return traces


# read the chunk range of the file (check the upper bound of chunk)
def chunk_read(infiles,chunk):
  traces = []
  counter = 1
  for infile in infiles:
    tempname  = 'temp%i.H'  %(tmp_num)
    tracename = 'trace%i.H' %(counter)
    # why does min2 and max2 not work? probably with attaching the header properly
    # copy the history file over
    copy1 = "cp %s %s" %(infile,tempname)
    print copy1
    commands.getoutput(copy1)
    # copy the history header file over
    copy2 ="cp %s@@ %s@@" %(infile,tempname)
    print copy2
    commands.getoutput(copy2)
    # get the correct chunk
    window1 = "Window < %s   min2=%d max2=%d > %s  " %(tempname,int(chunk),int(chunk),tracename)
    print window1
    commands.getoutput(window1)
    # get the correct header chunk
    window2 = "Window < %s@@ min2=%d max2=%d > %s@@" %(tempname,int(chunk),int(chunk),tracename)
    print window2
    commands.getoutput(window2)
    # attach the header
    attach = "echo \"hff=%s@@\" >> %s" %(tracename,tracename)
    print attach
    commands.getoutput(attach)
    # add trace name to list
    traces.append(tracename)
    # update the counter
    counter = counter+1
  return traces


# cat everything
def cat_files(fnames,target):
  block=20
  nfiles=len(fnames)

  count2=0
  while 1:
    total=len(fnames)
    cnames = []
    
    count1=0
    if (total//block>1):
      # For all complete blocks
      for ii in range(total//block):
        names = fnames[(count1)*block:(count1+1)*block]
        cname = 'temp%s_%i.H' %(str(count2),tmp_num)
        print "Cat3d %s axis=2 > %s" %(" ".join(names),cname)
        print commands.getstatusoutput("Cat3d max_memory=1000 %s axis=2 > %s" %(" ".join(names),cname))
        cnames.append(cname)
        for jj in names:
          commands.getstatusoutput("Rm %s %s@@"%(jj,jj))
        count1 = count1+1
        count2 = count2+1
        
      # For the remainder complete blocks
      if (total%block==1): # If only one file
        cname = fnames[count1*block]
        cnames.append(cname)
        
        count1 = count1+1
        count2 = count2+1

      if (total%block>1):
        names = fnames[count1*block:count1*block+total%block]
        cname = 'temp%s_%i.H' %(str(count2),tmp_num)
        print "Cat3d %s axis=2 > %s" %(" ".join(names),cname)
        print commands.getstatusoutput("Cat3d max_memory=1000 %s axis=2 > %s" %(" ".join(names),cname))
        cnames.append(cname)
        for jj in names:
          commands.getstatusoutput("Rm %s %s@@"%(jj,jj))
        count1 = count1+1
        count2 = count2+1
      
      # Repeat with new list
      fnames=cnames
      
    else:
      names = fnames
      print "Cat3d %s axis=2 > %s" %(" ".join(names),target)
      print commands.getstatusoutput("Cat3d max_memory=1000 %s axis=2 > %s" %(" ".join(names),target))
      cnames.append(target)
      for jj in names:
        commands.getstatusoutput("Rm %s %s@@"%(jj,jj))
      break



#------------------------------------------------------------------------------------------------#
# COMMANDS
#read the command line and associated files to be searched through
files,chunk,output_file=parse_command_line();
#print files
#print "output file is %s" %(output_file)
slimmed_files=remove_duplicates(files,chunk);
traces=chunk_read(slimmed_files,chunk)
#print traces
cat_files(traces,output_file)
