#! /usr/bin/env python

########################################################
### Python script for creating daily correlation stacks
########################################################


########################
# import libraries
########################
import commands
import sys
import os


########################
# some definitions
########################
SEPLIB = '/opt/SEP/SEP8.0/bin'
compressx = './Bin/jcCompressAxes.x'
computer = 'nanpu'

# flag for whether to run commands or just print
run = False
#run = True

# grab command line arguments
if len(sys.argv) !=2:
  print "Expecting one argument: [julian day]"
  sys.exit(0)
jday = int(sys.argv[1])


########################
# create file for each day
########################
# set output file name
outname = 'sh-ph2-down16-cube-day%s.H' %(jday)

# initialize a new list
tracenames = []
# loop over number of two-hour chunks in a day
for chunk in range(1,13):
  # grab tracename to be stacked
  tracename = 'sh-ph2-down16-%s-cube-si.H' %((jday*12)-8+chunk)
  tracenames.append(tracename)

### CONSTRUCT DAILY CORRELATION CUBE ###
# make string of files
traces = ' '.join(tracenames)
tempcube = 'tempcube_day%s.h' %(jday)
# make daily supercubes
cube_com = '%s/Cat3d axis=2 virtual=1 %s > %s' %(SEPLIB,traces,tempcube)
print cube_com
if run:
  commands.getstatusoutput(cube_com)

### SORT CORRELATION CUBE BY SOURCE AND RECEIVER LOCATIONS ###
# make sort file
tempsort = 'tempsort_day%s.h' %(jday)
# sort cubes by receiver
sort_com = '%s/Sort3d < %s nkeys=4 key1=receiver_line ng1=74 dg1=2 og1=1106 key2=receiver_point ng2=64 dg2=2 og2=4999 key3=source_line ng3=74 dg3=2 og3=1106 key4=source_point ng4=64 dg4=2 og4=4999 verb=1 > %s' %(SEPLIB,tempcube,tempsort)
print sort_com
if run:
  commands.getstatusoutput(sort_com)

### STACK SORTED CORRELATION CUBE ###  
# make stack file for data
tempstack = 'tempstack_day%s.h' %(jday)
# stack recordings at each receiver
stack_com = '%s/Stack3d < %s normalize=1 maxsize=10000 verb=1 > %s' %(SEPLIB,tempsort,tempstack)
print stack_com
if run:
  commands.getstatusoutput(stack_com)


### NOW HANDLE HEADER INFORMATION TO BE ATTACHED ###
# plan: grab sorted data file header, attached header to that header, sort and average that by header

### CREATE COPY OF HEADER FOR MANIPULATION ###
# make copy for headers
temphdr = 'temphdr_day%s.h' %(jday)
copyhdr_com = 'Cp %s@@ %s' %(tempsort,temphdr)
print copyhdr_com
if run:
  commands.getstatusoutput(copyhdr_com)
echohdr_com = 'echo "hff=%s@@" >> %s' %(tempsort,temphdr)
print echohdr_com
if run:
  commands.getstatusoutput(echohdr_com)

### SORT HEADER INFORMATION ###
# make sorted header 
tempsorthdr = 'temphdrsort_day%s.h' %(jday)
sorthdr_com = '%s/Sort3d < %s nkeys=4 key1=receiver_line ng1=74 dg1=2 og1=1106 key2=receiver_point ng2=64 dg2=2 og2=4999 key3=source_line ng3=74 dg3=2 og3=1106 key4=source_point ng4=64 dg4=2 og4=4999 verb=1 synch=1 > %s' %(SEPLIB,temphdr,tempsorthdr)
print sorthdr_com
if run:
  commands.getstatusoutput(sorthdr_com)

### STACK HEADER INFORMATION AND OUTPUT ###
stackhdr_com = '%s/Stack3d < %s normalize=1 maxsize=100000 verb=1 > %s@@' %(SEPLIB,tempsorthdr,temphdr)
print stackhdr_com
if run:
  commands.getstatusoutput(stackhdr_com)


### COMBINE THE DATA AND HEADER INFORMATION AGAIN, AND WINDOW EVERYTHING FOR STACKING ###
# call compress code
compressed = 'tempcompressed_day%s.h' %(jday)
compress_com = '%s < %s xy_in=%s@@ xy_out=%s@@ > %s' %(compressx,tempstack,temphdr,compressed,compressed)
print compress_com
if run:
  commands.getstatusoutput(compress_com)
# echo the headerinfo in
echof_com = 'echo "hff=%s@@" >> %s' %(compressed,compressed)
print echof_com
if run:
  commands.getstatusoutput(echof_com)
# more the header keys in
more_com = 'more ./Par/lb-si-hdr-info.p >> %s@@' %(compressed)
print more_com
if run:
  commands.getstatusoutput(more_com)
# window the files so that they can be catted later
copy = 'tempcopy_day%s.h' %(jday)
copy_com = '%s/Cp %s %s' %(SEPLIB,compressed,copy)
if run:
  commands.getstatusoutput(copy_com)
window_com = 'Window3d < %s synch=1 > %s' %(copy,outname)
print window_com
if run:
  commands.getstatusoutput(window_com)


#  ### CLEAN UP ###
#  clean_com1 = 'rm -f ./temp*day%s*.h*' %(jday)
#  print clean_com1
#  if run:
#    commands.getstatusoutput(clean_com1)
#  clean_com2 = 'rm -f /net/%s/scr1/jasonpc/temp*day%s*.h*' %(computer,jday)
#  print clean_com2
#  if run:
#    commands.getstatusoutput(clean_com2)
