program jcBlanking_paths
  ! Program for blanking outside of boundary of arrays
  ! Based on ray paths (i.e. where there was no update of slowness)
  ! Updated Aptril 2015
  !   - calculates average velocity of all model points outside of threshold
  !
  ! Inputs:
  !   1) history file of tomography results
  !   2) thresh: slowness threshold (all values below will be considered)
  !
  ! Outputs:
  !   1) history file containing coordinates of plots to be blanked
  !
  ! Jason Chang
  ! March 2015


  !=================================================
  ! standard declarations
  !=================================================
  use sep
  implicit none


  !=================================================
  ! declare variables
  !=================================================
  ! program variables
  integer                            :: ii,jj,ntraces,counter
  real                               :: xcoord,ycoord
  real                               :: avg

  ! input file variables
  integer                            :: n1,n2,n3
  real                               :: o1,o2,o3,d1,d2,d3
  real,allocatable,dimension(:,:)    :: datain

  ! slowness variables
  real                               :: thresh


  !=================================================
  ! open I/O
  !=================================================
  call sep_init()

  ! input history file parameters
  call from_history('n1',n1,1 )
  call from_history('n2',n2,1 )
  call from_history('n3',n3,1 )
  call from_history('o1',o1,0.)
  call from_history('o2',o2,0.)
  call from_history('o3',o3,0.)
  call from_history('d1',d1,1.)
  call from_history('d2',d2,1.)
  call from_history('d3',d3,1.)

  ! threshold parameter
  call from_param('thresh',thresh,0.)


  !=================================================
  ! initialize
  !=================================================
  allocate(datain(n1,n2))


  !=================================================
  ! kernel
  !=================================================
  ! read in data
  call sreed('in',datain,4*n1*n2)

  ! loop through rows of the model domain
  ntraces = 0
  avg = 0
  counter = 0
  do jj = 1,n2
    ycoord = o2+(jj-1)*d2
    do ii = 1,n1
      if (datain(ii,jj).lt.thresh) then
        xcoord = o1+(ii-1)*d1
        call srite('out',xcoord,4)
        call srite('out',ycoord,4)
        ntraces = ntraces+1
      else
        avg = avg+datain(ii,jj)
        counter = counter+1
      end if
    end do
  end do
  write(0,*) "average model value:",avg/counter


  !=================================================
  ! write out
  !=================================================
  call to_history('n1',2 )
  call to_history('o1',1.)
  call to_history('d1',1.)
  call to_history('n2',ntraces)
  call to_history('o2',1.     )
  call to_history('d2',1.     )
  call to_history('n3',1 )
  call to_history('o3',0.)
  call to_history('d3',1.)


  !=================================================
  ! clean up
  !=================================================
  deallocate(datain)
  call sep_close()
  call exit(0)


end program
