program jcExpandHeadersSI
  ! Program to add more parameters to the header file AFTER correlations
  ! Tailored toward Long Beach data
  !   ie. explicit receiver and source xy-loc and pt/line information
  !
  ! Inputs:
  !   1) receivers header information
  !   2) sources header information
  !
  ! Output:
  !   1) a new header file for correlations
  !        - ideally contains: receivers line/pts and xy-loc, sources line/pts, xy-loc, start time
  !   MAYBE) midpoint and azimuth information

  use sep
  implicit none

  ! declare variables
  integer                             :: ii
  integer                             :: nr1,nr2,ns1,ns2,nkeys
  integer                             :: rx_key,ry_key,rpoint_key,rline_key,t_start_key
  integer                             :: sx_key,sy_key,spoint_key,sline_key
  integer                             :: rx_ind,ry_ind,rpoint_ind,rline_ind,t_start_ind
  integer                             :: sx_ind,sy_ind,spoint_ind,sline_ind
  real,allocatable,dimension (:,:)    :: sheader,rheader,header

  ! open I/O
  call sep_init()
  
  if (exist_file('r_head')) then
    call from_aux('r_head','n1',nr1,1 )
    call from_aux('r_head','n2',nr2,1 )
  else
    call seperr('Need file containing receivers header info')
  end if

  if (exist_file('s_head')) then
    call from_aux('s_head','n1',ns1,1 )
    call from_aux('s_head','n2',ns2,1 )
  else
    call seperr('Need file containing sources header info')
  end if

  ! get the indices of the keys you care about
  call from_param('rx_ind'     ,rx_ind     ,1)
  call from_param('ry_ind'     ,ry_ind     ,2)
  call from_param('rline_ind'  ,rline_ind  ,7)
  call from_param('rpoint_ind' ,rpoint_ind ,8)
  call from_param('sx_ind'     ,sx_ind     ,1)
  call from_param('sy_ind'     ,sy_ind     ,2)
  call from_param('sline_ind'  ,sline_ind  ,7)
  call from_param('spoint_ind' ,spoint_ind ,8)
  call from_param('t_start_ind',t_start_ind,9)

  ! calculate the number of keys you care about
  call from_param('nkeys'      ,nkeys      ,9)
  call from_param('rx_key'     ,rx_key     ,1)
  call from_param('ry_key'     ,ry_key     ,2)
  call from_param('rline_key'  ,rline_key  ,3)
  call from_param('rpoint_key' ,rpoint_key ,4)
  call from_param('sx_key'     ,sx_key     ,5)
  call from_param('sy_key'     ,sy_key     ,6)
  call from_param('sline_key'  ,sline_key  ,7)
  call from_param('spoint_key' ,spoint_key ,8)
  call from_param('t_start_key',t_start_key,9)

  call to_history('n1',nkeys  )
  call to_history('n2',nr2*ns2)

  call sep_close()

  ! allocate
  allocate(rheader(nr1 ,nr2))
  allocate(sheader(ns1 ,ns2))
  allocate(header(nkeys,nr2))

  ! read in
  call sreed("r_head",rheader,4*nr1*nr2)
  call sreed("s_head",sheader,4*ns1*ns2)

  ! fill header with appropriate info
  do ii=1,ns2
    ! copy over header info
    header(rx_key     ,:) = rheader(rx_ind     ,: )
    header(ry_key     ,:) = rheader(ry_ind     ,: )
    header(rpoint_key ,:) = rheader(rpoint_ind ,: )
    header(rline_key  ,:) = rheader(rline_ind  ,: )
    header(sx_key     ,:) = sheader(sx_ind     ,ii)
    header(sy_key     ,:) = sheader(sy_ind     ,ii)
    header(spoint_key ,:) = sheader(spoint_ind ,ii)
    header(sline_key  ,:) = sheader(sline_ind  ,ii)
    header(t_start_key,:) = sheader(t_start_ind,ii)

    ! write out
    call srite('out',header,4*nkeys*nr2)
  end do

  ! clean up
  deallocate(rheader,sheader,header)


end program
