module symmetrize_mod


  !=====================================
  ! standard declarations
  !=====================================
  implicit none


  !=====================================
  ! declare module variables
  !=====================================
  integer                        :: n1,nt
  private
  public :: symmetrize_init,symmetrize_close,symmetrize_fold,symmetrize_unfold,symmetrize_fold_cmplx

  contains




  !=====================================
  ! initialization subroutine
  !=====================================
  subroutine symmetrize_init(n1_in)

    !=============================
    ! declare variables
    !=============================
    integer     :: n1_in
    !real        :: o1_in,d1_in
    !real        :: ot

    !=============================
    ! set these variables for entire module
    !=============================
    n1 = n1_in;  !o1 = o1_in;  d1 = d1_in;
    !n2 = n2_in;  o2 = o2_in;  d2 = d2_in;
    !n3 = n3_in;  o3 = o3_in;  d3 = d3_in;

    !=============================
    ! set other parameters
    !=============================
    nt = (n1/2)+1

  end subroutine




  !=====================================
  ! symmetrize module cleanup
  !=====================================
  subroutine symmetrize_close()
  end subroutine




  !=====================================
  ! folded symmetrize subroutine
  !=====================================
  subroutine symmetrize_fold(dataout,datain)

    !=============================
    ! declare variables
    !=============================
    integer             :: ii
    real,dimension (:)  :: datain,dataout

    !=============================
    ! kernel
    !=============================
    dataout(1) = datain(nt)
    do ii=1,nt-2
      dataout(1+ii) = (datain(nt+ii)+datain(nt-ii))/2.
    end do
    dataout(nt) = datain(1)

  end subroutine


  !=====================================
  ! folded symmetrize subroutine for complex input
  !=====================================
  subroutine symmetrize_fold_cmplx(dataout,datain)

    !=============================
    ! declare variables
    !=============================
    integer                :: ii
    real                   :: rpart1,rpart2,ipart1,ipart2
    complex,dimension (:)  :: datain,dataout

    !=============================
    ! kernel
    !=============================
    dataout(1) = datain(nt)
    do ii=1,nt-2
      rpart1 = real(datain(nt+ii))
      rpart2 = real(datain(nt-ii))
      ipart1 = aimag(datain(nt+ii))
      ipart2 = aimag(datain(nt-ii))
      dataout(1+ii) = cmplx((rpart1+rpart2)/2,(ipart1-ipart2)/2)
    end do
    dataout(nt) = datain(1)

  end subroutine




  !=====================================
  ! unfolded symmetrize subroutine
  !=====================================
  subroutine symmetrize_unfold(dataout,datain)
    
    !=============================
    ! declare variables
    !=============================
    integer             :: ii
    real,dimension (:)  :: datain,dataout

    !=============================
    ! kernel
    !=============================
    dataout(nt) = datain(nt)
    do ii=1,nt-2
      dataout(nt+ii) = (datain(nt+ii)+datain(nt-ii))/2.
      dataout(nt-ii) = (datain(nt+ii)+datain(nt-ii))/2.
    end do
    dataout(1) = datain(1)

  end subroutine
  

end module
