module taper_mod
  ! Module containing various types of tapers
  !
  ! Jason Chang
  ! April 2014

  !==========================================================
  ! includes
  !==========================================================
  implicit none


  !==========================================================
  ! declare module variables
  !==========================================================
  integer,private                           :: n1,opt
  real,private                              :: o1,d1
  real,private,parameter                    :: pi=3.1415926535
  real,private,allocatable,dimension (:)    :: taper

  contains


  !==========================================================
  ! gaussian initialization subroutine
  !==========================================================
  subroutine taper_init(n1_in,o1_in,d1_in,opt_in)

    !===========================================
    ! declare variables
    !===========================================
    integer                    :: ii
    integer                    :: n1_in,opt_in
    real                       :: o1_in,d1_in

    !===========================================
    ! pass variables into entire modules
    !===========================================
    n1 = n1_in
    o1 = o1_in
    d1 = d1_in
    opt = opt_in

    !===========================================
    ! allocate arrays
    !===========================================
    allocate(taper(n1)); taper=0

  end subroutine


  !===========================================================
  ! close taper subroutine
  !===========================================================
  subroutine taper_close()

    !===========================================
    ! deallocate input and output arrays
    !===========================================
    deallocate(taper)

  end subroutine


  !===========================================================
  ! subroutine to build tapers
  !===========================================================
  subroutine apply_taper(dat,t1,t2,t3,t4)

    !===========================================
    ! declare variables
    !===========================================
    integer                           :: ii
    integer                           :: t1_ind,t2_ind,t3_ind,t4_ind
    real                              :: t1,t2,t3,t4
    real,dimension (:)                :: dat

    !===========================================
    ! set time indices
    !===========================================
    t1_ind = nint((t1-o1)/d1)+1
    t2_ind = nint((t2-o1)/d1)+1
    t3_ind = nint((t3-o1)/d1)+1
    t4_ind = nint((t4-o1)/d1)+1

    !============================================
    ! construct filter
    !============================================
    select case(opt)

      !==============================
      ! hann-half filter
      !==============================
      case(0)
        taper(1:t1_ind) = 1.
        do ii=t1_ind,t2_ind
          taper(ii)= .5*(1.+cos(pi*(ii-t1_ind)/real(t2_ind-t1_ind)))
        end do

      !==============================
      ! hann filter
      !==============================
      case(1)
        do ii=t1_ind,t2_ind
          taper(ii)= .5*(1.-cos(2.*pi*(ii-t1_ind)/real(t2_ind-t1_ind)))
        end do

      !==============================
      ! hann-band filter
      !==============================
      case(2)
        do ii=t1_ind,t2_ind
          taper(ii)= .5*(1.-cos(pi*(ii-t1_ind)/real(t2_ind-t1_ind)))
        end do
        taper(t2_ind:t3_ind) = 1.
        do ii=t3_ind,t4_ind
          taper(ii)= .5*(1.+cos(pi*(ii-t3_ind)/real(t4_ind-t3_ind)))
        end do

    end select

    dat = dat*taper

  end subroutine

end module
